"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();
/* eslint no-param-reassign:0 */


exports.default = function (schema) {
  return function flowRelayQueryPlugin(plugin) {
    var t = plugin.types;

    var activeVisitor = {
      TypeAlias: function TypeAlias(_ref, state) {
        var node = _ref.node;
        // eslint-disable-line no-shadow
        if (node.type !== "TypeAlias") {
          return;
        }

        if (node.right.type === "ObjectTypeAnnotation") {
          state.flowTypes[node.id.name] = node.right;
        }
      },
      JSXElement: function JSXElement(_ref2, state) {
        var node = _ref2.node;
        // eslint-disable-line no-shadow
        if (node.type !== "JSXElement") {
          return;
        }

        var name = node.openingElement.name.name;
        var filename = state.imports[name];
        if (filename) {
          var _visitor = {
            CallExpression: function CallExpression(p) {
              if ((0, _react.isRelayCreateContainer)(p) && p.node.type === "CallExpression") {
                var n = p.node;

                var _n$arguments = _slicedToArray(n.arguments, 2),
                    arg1 = _n$arguments[0],
                    arg2 = _n$arguments[1];

                if (arg1.type !== "Identifier" || arg2.type !== "ObjectExpression") {
                  return;
                }

                var reactComponentName = arg1.name;
                var fragment = arg2.properties.find(function (_) {
                  return _.key.type === "Identifier" && _.key.name === "fragments";
                });
                var fragmentNames = [];
                if (fragment && fragment.value.type === "ObjectExpression") {
                  fragmentNames = fragment.value.properties.reduce(function (fragNames, property) {
                    if (property.key.type === "Identifier") {
                      fragNames.push(property.key.name);
                    }
                    return fragNames;
                  }, []);
                }
                state.relayContainerFragments[reactComponentName] = fragmentNames;
              }
            }
          };
          (0, _parse.parseFile)(filename, _visitor, state);
        }
      },
      ClassDeclaration: function ClassDeclaration(_ref3, state) {
        var node = _ref3.node;
        // eslint-disable-line no-shadow
        if (node.type !== "ClassDeclaration") {
          return;
        }

        var _ref4 = (0, _react.parseReactComponentClass)(node) || {},
            className = _ref4.className,
            propType = _ref4.propType;

        if (className && propType) {
          state.componentPropTypes[className] = propType;
        }
      },
      ArrowFunctionExpression: function ArrowFunctionExpression(_ref5, state) {
        var node = _ref5.node,
            parent = _ref5.parent;
        // eslint-disable-line no-shadow
        if (node.type !== "ArrowFunctionExpression") {
          return;
        }

        var functionName = parent && parent.id && parent.id.name;

        if (functionName && node.params.length === 1) {
          var typeAnnotation = node.params[0].typeAnnotation;
          typeAnnotation = typeAnnotation && typeAnnotation.typeAnnotation;

          if (typeAnnotation && typeAnnotation.type === "GenericTypeAnnotation") {
            state.componentPropTypes[functionName] = typeAnnotation.id.name;
          }
        }
      },


      // is this the fragment generate marker function
      CallExpression: function CallExpression(path, state) {
        var node = path.node,
            parent = path.parent;

        if (node.type !== "CallExpression") {
          return;
        }

        if (node.callee.name !== GEN_FRAG_FROM_PROPS_NAME) {
          return;
        }

        var relayCreateContainer = path.findParent(function (p) {
          return t.isCallExpression(p) && (0, _react.isRelayCreateContainer)(p);
        });
        if (!relayCreateContainer || relayCreateContainer.node.type !== "CallExpression") {
          return;
        }

        var argPaths = relayCreateContainer.get("arguments");
        var arg1Path = Array.isArray(argPaths) && argPaths[0];
        if (!arg1Path) {
          throw new Error("Could not find first argument for Relay.createContainer");
        }

        var reactComponentNames = [];
        if (arg1Path.node.type === "Identifier") {
          reactComponentNames.push(arg1Path.node.name);
        } else {
          arg1Path.traverse({
            Identifier: function Identifier(identifierPath) {
              if (identifierPath.node.type === "Identifier") {
                reactComponentNames.push(identifierPath.node.name);
              }
            }
          });
        }
        var typeName = reactComponentNames.reduce(function (result, reactComponentName) {
          return result || state.componentPropTypes[reactComponentName];
        }, null);
        if (!typeName) {
          var reactComponentNamesStr = reactComponentNames.join(", ");
          throw new Error("Could not find flow prop types for possible react components [" + reactComponentNamesStr + "]");
        }

        var type = state.flowTypes[typeName];
        if (!type) {
          throw new Error("There is no flow type object with name " + typeName);
        }

        var fragmentKey = parent && t.isProperty(parent) && parent.key.name;
        var typeAtKey = type.properties.find(function (_) {
          return _.key.name === fragmentKey;
        });
        if (!typeAtKey) {
          throw new Error("There is no property named '" + fragmentKey + "' in flow type " + typeName);
        }

        var generateFragmentOptions = (0, _fragmentOptions.calculateFragmentOptions)(node);
        var fragmentName = generateFragmentOptions.name || uppercaseFirstChar(typeAtKey.key.name);
        var fragmentDirectives = generateFragmentOptions.directives || {};

        (0, _graphql.checkPropsObjectTypeMatchesSchema)(schema, fragmentName, typeAtKey);
        var graphQlQuery = (0, _graphql.toGraphQLQueryString)(fragmentName, fragmentDirectives, typeAtKey, state.relayContainerFragments);

        // relay needs a start location on the Relay.QL template string
        var start = node.loc.start;
        path.replaceWithSourceString(graphQlQuery);
        if (path.node.type === "ArrowFunctionExpression") {
          path.node.body.loc = { start: start };
        }
      }
    };

    var importVisitor = {
      ImportDeclaration: function ImportDeclaration(path, state) {
        var node = path.node;
        if (node.type !== "ImportDeclaration") {
          return;
        }

        var _ref6 = (0, _import.parseImport)(node, state.filename) || {},
            filename = _ref6.filename,
            variables = _ref6.variables;

        if (filename) {
          if ((0, _import.isTypeImport)(node)) {
            var _visitor2 = {
              TypeAlias: function TypeAlias(_ref7) {
                var n = _ref7.node;
                // eslint-disable-line no-shadow, no-unused-vars
                if (n.type !== "TypeAlias") {
                  return;
                }

                var typeName = n.id.name;
                if (variables.indexOf(typeName) >= 0 && n.right.type === "ObjectTypeAnnotation") {
                  state.flowTypes[typeName] = n.right;
                }
              }
            };
            (0, _parse.parseFile)(filename, _visitor2, {});
          } else {
            variables.forEach(function (variable) {
              state.imports[variable] = filename;
            });
          }
        }

        // remove the marker function import
        if (node.specifiers.find(function (_) {
          return _.local.name === GEN_FRAG_FROM_PROPS_NAME;
        })) {
          path.remove();
        }
      }
    };

    return {
      visitor: {
        Program: function Program(path, state) {
          var imports = {};
          var flowTypes = {};
          var filename = state.file.opts.filename;
          path.traverse(importVisitor, { imports: imports, flowTypes: flowTypes, filename: filename });

          if (imports[GEN_FRAG_FROM_PROPS_NAME]) {
            path.traverse(activeVisitor, {
              imports: imports,
              flowTypes: flowTypes,
              componentPropTypes: {},
              relayContainerFragments: {}
            });
          }
        }
      }
    };
  };
};

var _import = require("./utils/import");

var _react = require("./utils/react");

var _parse = require("./utils/parse");

var _fragmentOptions = require("./utils/fragmentOptions");

var _graphql = require("./utils/graphql");

var GEN_FRAG_FROM_PROPS_NAME = "generateFragmentFromProps";

var uppercaseFirstChar = function uppercaseFirstChar(str) {
  return str.charAt(0).toUpperCase() + str.slice(1);
};