"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.calculateFragmentOptions = calculateFragmentOptions;


function objectExpressionProperties(node) {
  return node.properties.reduce(function (opts, property) {
    if (property.key.type === "Identifier") {
      opts[property.key.name] = property.value;
    }
    return opts;
  }, {});
}
/* eslint no-param-reassign:0 */
function calculateFragmentOptions(genFragmentMarkerFunction) {
  var options = {};

  var optionObj = genFragmentMarkerFunction.arguments[0];
  if (optionObj && optionObj.type === "ObjectExpression") {
    var fragmentOptions = objectExpressionProperties(optionObj);

    var nameNode = fragmentOptions.name;
    if (nameNode && nameNode.type === "StringLiteral") {
      options.name = nameNode.value;
    }

    var directivesNode = fragmentOptions.directives;
    if (directivesNode && directivesNode.type === "ObjectExpression") {
      (function () {
        var rawDirectiveOptions = objectExpressionProperties(directivesNode);
        options.directives = Object.keys(rawDirectiveOptions).reduce(function (opts, directiveName) {
          var directiveArgsNode = rawDirectiveOptions[directiveName];
          if (directiveArgsNode.type === "ObjectExpression") {
            // find any arguments that go with the directive
            opts[directiveName] = directiveArgsNode.properties.reduce(function (args, property) {
              if (property.key.type === "Identifier") {
                var argName = property.key.name;
                switch (property.value.type) {
                  case "BooleanLiteral":
                  case "NumericLiteral":
                  case "StringLiteral":
                    args[argName] = property.value.value;
                    break;
                  default:
                  // nothing
                }
              }
              return args;
            }, {});
          }
          return opts;
        }, {});
      })();
    }
  }

  return options;
}